// DlgWinMgr.cpp : implementation file
// COPYRIGHT (C) 1999-2000 by Mike Melnikov zmike@andnow.ru
//

#include "stdafx.h"

#include "Commands.h"
#include "DlgWinMgr.h"
#include "Utils.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

using namespace std;

/////////////////////////////////////////////////////////////////////////////
// CDlgWinMgr dialog

static int COLTIME = -1;
static int COLEXT = -1;
static int COLDIRNAME = -1;

BEGIN_MESSAGE_MAP(CDlgWinMgr, CSortMgr)
  ON_WM_CONTEXTMENU()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDlgWinMgr message handlers
void CDlgWinMgr::OnInit()
{
	m_ctrl.InsertColumn(COL_FILENAME, "File");
  COLTIME = COLEXT = COLDIRNAME = 1;
  if(CCommands::getValue("Time",1))
  {
  	m_ctrl.InsertColumn(COL_TIME, "Time");
    COLEXT++;COLDIRNAME++;
  }
  else
  {
    COLTIME = 5;
  }
	if(CCommands::getValue("Extension",1))
  {
    m_ctrl.InsertColumn(COL_EXT, "Extension");
    COLDIRNAME++;
  }
  else
  {
    COLEXT = 5;
  }
	if(CCommands::getValue("Directory",1))
    m_ctrl.InsertColumn(COL_DIRNAME, "Directory");
  else
  {
    COLDIRNAME = 5;
  }
  m_ctrl.SendMessage(LVM_SETEXTENDEDLISTVIEWSTYLE,0,(LPARAM)LVS_EX_FULLROWSELECT);
}

void CDlgWinMgr::SelectCurrent()
{
	CString strFullPath = CCommands::Get()->GetCurrentFile();
  baseFilename(strFullPath,strFullPath,CString());
  LVFINDINFO info;
  info.flags = LVFI_STRING;
  info.psz = LPCTSTR(strFullPath);
  int ind = m_ctrl.FindItem(&info);
  if(ind>=0)
    BOOL res = m_ctrl.SetItemState(ind,LVIS_SELECTED|LVIS_FOCUSED,-1);
}

void CDlgWinMgr::FillList()
{
	int i = 0;
	m_ctrl.DeleteAllItems();
  for(list<CFileItem*>::iterator  it = m_List.begin();it != m_List.end();++it)
	{
		CFileItem* pFileItem = *it;
		CString strFile = pFileItem->m_File;
		m_ctrl.InsertItem(i, strFile, (LPARAM)pFileItem);
		m_ctrl.SetItemText(i, COLDIRNAME, pFileItem->m_Dir);
		m_ctrl.SetItemText(i,COLTIME, pFileItem->m_timeLast.Format(_T("%I:%M:%S %p")));
		m_ctrl.SetItemText(i,COLEXT,findExtension(strFile));
		i++;
	}
	m_ctrl.SetWidths();
}

int CDlgWinMgr::Sort(LPARAM lParam1, LPARAM lParam2)
{
	int iColumn = abs(m_ctrl.m_iSortedCol)-1; // iColumn here is "1 based" so we don't get -0
	int iComp = 0;
	CFileItem* pFIOne = (CFileItem *)lParam1;
	CFileItem* pFITwo = (CFileItem *)lParam2;
	if(iColumn==COL_FILENAME)
	{
		iComp = pFIOne->m_File.CompareNoCase(pFITwo->m_File);
  }
  else if(iColumn==COLDIRNAME)
  {
		iComp = pFIOne->m_Dir.CompareNoCase(pFITwo->m_Dir);
    if(iComp==0)
    {
			iComp = pFIOne->m_File.CompareNoCase(pFITwo->m_File);
    }
  }
  else if(iColumn==COLTIME)
  {
    CTime timeOne = pFIOne->m_timeLast;
	CTime timeTwo = pFITwo->m_timeLast;
	iComp = timeOne > timeTwo ? 1 : ( timeOne == timeTwo ? 0 : -1);
  }
  else if(iColumn==COLEXT)
  {
		iComp = findExtension(pFIOne->m_File).CompareNoCase(findExtension(pFITwo->m_File));
	}
	if(m_ctrl.m_iSortedCol < 0)
	{
		iComp = -iComp; // reverse sorting sense.
	}
	return iComp;
}


void CDlgWinMgr::OnDel()
{
  vector<CString> docs;
	for( CListSelIterator iter(m_ctrl);iter;)
	{
		CString strDoc = m_ctrl.GetItemText(iter,COLDIRNAME);
    addSlash(strDoc);
		strDoc += m_ctrl.GetItemText(iter,COL_FILENAME);
    docs.push_back(strDoc);
	}

  vector<CString>::reverse_iterator it;
  for (it = docs.rbegin(); it != docs.rend();it++)
		CCommands::Get()->CloseDoc(*it);

	m_ctrl.DoSort();
  Update();
}
void CDlgWinMgr::OnSelect(CMListCtrl& ctrl)
{
	for( CListSelIterator iter(ctrl);iter;)
	{
		CFileItem *pItem = (CFileItem *)ctrl.GetItemData(iter);
		ASSERT(pItem);
		CCommands::Get()->OpenDoc(contact(pItem->m_Dir,pItem->m_File));
	}
  CDlgMgr::OnSelect(ctrl);
}



//#include <shlobj.h>
//#include "..\..\ZSpliter\WndTabs225Src\BCMenu.h"
//#include "..\..\ZSpliter\WndTabs225Src\ShellContextMenu.h"
#include "comdef.h"



LPITEMIDLIST GetNextItem (LPITEMIDLIST pidl)
{
    USHORT nLen;

    if ((nLen = pidl->mkid.cb) == 0)
        return NULL;
    
    return (LPITEMIDLIST) (((LPBYTE) pidl) + nLen);
}

UINT GetItemCount (LPITEMIDLIST pidl)
{
    USHORT nLen;
    UINT nCount;

    nCount = 0;
    while ((nLen = pidl->mkid.cb) != 0) {
        pidl = GetNextItem (pidl);
        nCount++;
    }
    return nCount;
}

LPITEMIDLIST DuplicateItem (LPMALLOC pMalloc, LPITEMIDLIST pidl)
{
    USHORT nLen;
    LPITEMIDLIST pidlNew;

    nLen = pidl->mkid.cb;
    if (nLen == 0)
        return NULL;

    pidlNew = (LPITEMIDLIST) pMalloc->Alloc (nLen + sizeof (USHORT));
    if (pidlNew == NULL)
        return NULL;

    CopyMemory (pidlNew, pidl, nLen);
    *((USHORT*) (((LPBYTE) pidlNew) + nLen)) = 0;

    return pidlNew;
}


BOOL DoExplorerMenu (HWND hwnd, LPCTSTR pszPath, POINT point)
{
    LPMALLOC pMalloc;
    LPSHELLFOLDER psfFolder, psfNextFolder;
    LPITEMIDLIST pidlMain, pidlItem, pidlNextItem;
    ITEMIDLIST **ppidl;
    LPCONTEXTMENU pContextMenu;
    CMINVOKECOMMANDINFO ici;
    ULONG ulCount, ulAttr;
    TCHAR tchPath[MAX_PATH];
    WCHAR wchPath[MAX_PATH];    
    UINT nCount, nCmd;
    BOOL bResult;
    HMENU hMenu;

    //
    // Make sure the file name is fully qualified and in Unicode format.
    //
    GetFullPathName (pszPath, sizeof (tchPath) / sizeof (TCHAR), tchPath,
        NULL);

    if (IsTextUnicode (tchPath, lstrlen (tchPath), NULL))
        lstrcpy ((char *) wchPath, tchPath);
    else
        MultiByteToWideChar (CP_ACP, 0, pszPath, -1, wchPath,
            sizeof (wchPath) / sizeof (WCHAR));

    //
    // Get pointers to the shell's IMalloc interface and the desktop's
    // IShellFolder interface.
    //
    bResult = FALSE;

    if (!SUCCEEDED (SHGetMalloc (&pMalloc)))
        return bResult;

    if (!SUCCEEDED (SHGetDesktopFolder (&psfFolder))) {
        pMalloc->Release ();
        return bResult;
    }

    //
    // Convert the path name into a pointer to an item ID list (pidl).
    //
    if (SUCCEEDED (psfFolder->ParseDisplayName (hwnd,
        NULL, wchPath, &ulCount, &pidlMain, &ulAttr)) && (pidlMain != NULL)) {

        if (nCount = GetItemCount (pidlMain)) { // nCount must be > 0
            //
            // Initialize psfFolder with a pointer to the IShellFolder
            // interface of the folder that contains the item whose context
            // menu we're after, and initialize pidlItem with a pointer to
            // the item's item ID. If nCount > 1, this requires us to walk
            // the list of item IDs stored in pidlMain and bind to each
            // subfolder referenced in the list.
            //
            pidlItem = pidlMain;

            while (--nCount) {
                //
                // Create a 1-item item ID list for the next item in pidlMain.
                //
                pidlNextItem = DuplicateItem (pMalloc, pidlItem);
                if (pidlNextItem == NULL) {
                    pMalloc->Free (pidlMain);
                    psfFolder->Release ();
                    pMalloc->Release ();
                    return bResult;
                }

                //
                // Bind to the folder specified in the new item ID list.
                //
                if (!SUCCEEDED (psfFolder->BindToObject (pidlNextItem, NULL, IID_IShellFolder, (void **)&psfNextFolder))) {
                    pMalloc->Free (pidlNextItem);
                    pMalloc->Free (pidlMain);
                    psfFolder->Release ();
                    pMalloc->Release ();
                    return bResult;
                }

                //
                // Release the IShellFolder pointer to the parent folder
                // and set psfFolder equal to the IShellFolder pointer for
                // the current folder.
                //
                psfFolder->Release ();
                psfFolder = psfNextFolder;

                //
                // Release the storage for the 1-item item ID list we created
                // just a moment ago and initialize pidlItem so that it points
                // to the next item in pidlMain.
                //
                pMalloc->Free (pidlNextItem);
                pidlItem = GetNextItem (pidlItem);
            }

            //
            // Get a pointer to the item's IContextMenu interface and call
            // IContextMenu::QueryContextMenu to initialize a context menu.
            //
            ppidl = &pidlItem;
            if (SUCCEEDED (psfFolder->GetUIObjectOf (hwnd, 1, (const ITEMIDLIST **)ppidl, IID_IContextMenu, NULL, (void **)&pContextMenu))) {

                hMenu = CreatePopupMenu ();
                if (SUCCEEDED (pContextMenu->QueryContextMenu (hMenu, 0, 1, 0x7FFF, CMF_EXPLORE))) {

                    ClientToScreen (hwnd, &point);

                    //
                    // Display the context menu.
                    //
                    nCmd = TrackPopupMenu (hMenu, TPM_LEFTALIGN |
                        TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_RETURNCMD,
                        point.x, point.y, 0, hwnd, NULL);

                    //
                    // If a command was selected from the menu, execute it.
                    //
                    if (nCmd) {
                        ici.cbSize          = sizeof (CMINVOKECOMMANDINFO);
                        ici.fMask           = 0;
                        ici.hwnd            = hwnd;
                        ici.lpVerb          = MAKEINTRESOURCE (nCmd - 1);
                        ici.lpParameters    = NULL;
                        ici.lpDirectory     = NULL;
                        ici.nShow           = SW_SHOWNORMAL;
                        ici.dwHotKey        = 0;
                        ici.hIcon           = NULL;

                        if (SUCCEEDED (
                            pContextMenu->InvokeCommand (&ici)))
                            bResult = TRUE;
                    }
                }
                DestroyMenu (hMenu);
                pContextMenu->Release ();
            }
        }
        pMalloc->Free (pidlMain);
    }

    //
    // Clean up and return.
    //
    psfFolder->Release ();
    pMalloc->Release ();

    return bResult;
}


void CDlgWinMgr::OnContextMenu( CWnd* pWnd, CPoint pos )
{
  if(&m_ctrl!=pWnd)
    return;
  UINT uFlags;
  ScreenToClient(&pos);
  int nItem = m_ctrl.HitTest(pos, &uFlags);
  if(nItem==-1)
    return;
  CFileItem *pItem = (CFileItem *)m_ctrl.GetItemData(nItem);
  if(!pItem)
    return;

  DoExplorerMenu(*pWnd,pItem->m_Dir+"\\"+pItem->m_File,pos);
  return;
}
