// TagsListDP.cpp : implementation file
// COPYRIGHT (C) 1999-2000 by Mike Melnikov zmike@andnow.ru
//
#include "stdafx.h"

#include "Commands.h"
#include "TagsListDP.h"
#include "tags\entry.h"
#include "tags\options.h"

#include <algorithm>

#include "graph\link.h"
#include "graph\nodes.h"
#include "graph\linkpoint.h"
#include "graph\node.h"
#include "graph\links.h"
#include "graph\AddflowConst.h"
#include "graph\linkpoints.h"
#include "graph\picture.h"

extern tagFile TagFile;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CLink addLink(CNodes& nodes,int x,int y)
{
	CLink link;
  VARIANT vaIdx, vaIdx2;
  VariantInit(&vaIdx);
  VariantInit(&vaIdx2);
  vaIdx.vt = VT_I4;
  vaIdx2.vt = VT_I4;	

  vaIdx.lVal = x + 1;
  vaIdx2.lVal = y + 1;
  link = nodes.Item(vaIdx).GetOutLinks().Add(nodes.Item(vaIdx2));
  return link;
}


static int width = 2000;
static int height = 220;
static int dWidth = 600;
static int dHeight = 150;

void addNode(CNodes& nodes,int x,int y,const char* name)
{
  CString s;
  if(x<0)
  {
    s.Format("%d %d",x,y);
    OutputDebugString(s);
    x = 0;
  }
	CNode node = nodes.Add(x,y, width, height);
  char* n = strrchr(name,'\\');
  node.SetText(n ? n+1 : name);
  node.SetToolTip(name);
  node.SetFillColor(RGB(255, 255, 0));
}

class Link
{
 public:
   Link(int ai1,int ai2) : i1(ai1),i2(ai2){}
  int i1,i2;
};

GraphClasses classes;
vector<Link>  links;
int counts[100] = {0};
int maxI = -1;
int Maxlevel = -1;
int countsInFraph[100] = {0};

void TagsListDP::Select(CString& str,GraphClasses& cl)
{
  int count = cl.size();
  for(int i=0;i<count;i++)
    if(cl[i].m_name==str)
    {
      CNodes& nodes = m_AddFlow.GetNodes();
      VARIANT vaIdx;
      VariantInit(&vaIdx);
      vaIdx.vt = VT_I4;
      vaIdx.lVal = i+1;
      CNode node = nodes.Item(vaIdx);
      float l = node.GetLeft();
      float t = node.GetTop();
      m_AddFlow.SetXScroll(l);
      m_AddFlow.SetYScroll(t);
      break;
    }
}

void TagsListDP::Do_Select(CString str)
{
  if(m_processFocus)
    Select(str,classes);
}

bool* recursiveMask = NULL;

void update(int idx)
{
  // check recursive
  if(recursiveMask[idx])
    return;
  recursiveMask[idx] = true;

  GraphClass& cl = classes[idx]; 
  if(cl.m_level<0)
  {
    cl.m_level = 0;
  }
  for(int i=0;i<cl.m_ichildren.size();i++)
  {
    GraphClass& child = classes[cl.m_ichildren[i]]; 

    int oldLEvel = child.m_level;
    int newLevel = cl.m_level + 1;

    if(newLevel>oldLEvel)
    {
      Maxlevel = max(Maxlevel,newLevel);
      child.m_level = newLevel;
    }

    update(cl.m_ichildren[i]);
  }
}

void do_update(int idx)
{
  // I don't know how to make it faster
  memset(recursiveMask,false,classes.size()*sizeof(bool)); 
  update(idx);
}


void setTop(CNodes& nodes,int qqq,bool isFirst)
{
  GraphClass& cl = classes[qqq];
  if(!cl.setRightPosition)
  {
    VARIANT vaIdx;
    VariantInit(&vaIdx);
    vaIdx.vt = VT_I4;
    vaIdx.lVal = qqq+1;
    CNode node = nodes.Item(vaIdx);
  
    int newI = max(maxI + (isFirst ? 0 : 1),counts[cl.m_level]+1);
    counts[cl.m_level] = newI;
    maxI = newI;
    cl.row = newI;
    node.SetTop((dHeight+height)*newI);
  
    cl.setRightPosition = true;
  }
}

void addLink(CNodes& nodes,int qqq,bool isFirst)
{
  GraphClass& cl = classes[qqq];

  if(!cl.addChildrens)
  {
    cl.addChildrens = true;

    setTop(nodes,qqq,isFirst);

    int count = cl.m_ichildren.size();
    int already = 0;
    int lev = 1;
    while(already<count)
    {
      for(int j=0;j<count;j++)
      {
        int idx = cl.m_ichildren[j];
        GraphClass& child = classes[idx];
        if(child.m_level == cl.m_level+lev)
        {
          addLink(nodes,idx,already==0);
          already++;
          links.push_back(Link(qqq,idx));
        }
        else if(child.m_level <= cl.m_level)
          already++;
      }
      lev++;
    }
  }
}

void draw(CNodes& nodes)
{
  for(int q=0;q<classes.size();q++)
  {
    addNode(nodes,classes[q].m_level*(width+dWidth),0,classes[q].m_name);
  }
  
  for(int qq=0;qq<classes.size();qq++)
  {
    if(classes[qq].m_level==0)
      addLink(nodes,qq,qq==0);
  }
    
  for(int l=0;l<links.size();l++)
  {
    CLink link = addLink(nodes,links[l].i1,links[l].i2);

    link.SetArrowDst(afNoArrow);

    GraphClass& c1 = classes[links[l].i1];
    GraphClass& c2 = classes[links[l].i2];

    if(c1.m_level+1 == c2.m_level && c1.row==c2.row)
    {
      // do nothing
    }
    else if(c1.m_level+1 == c2.m_level) // c2 second .. child
    {
      link.SetLinkStyle(afHVH);
    }
    else // c1 second .. parent
    {
      //link.SetArrowMid(afFilledArrow15);
      link.SetLinkStyle(afHVHV);

  	  CLinkPoints points = link.GetExtraPoints();

      ASSERT(points.GetCount()==5);
      CLinkPoint lp0 = link.GetExtraPoints().GetItem(0);
      CLinkPoint lp1 = link.GetExtraPoints().GetItem(1);
      CLinkPoint lp2 = link.GetExtraPoints().GetItem(2);
      CLinkPoint lp3 = link.GetExtraPoints().GetItem(3);
      CLinkPoint lp4 = link.GetExtraPoints().GetItem(4);

      int x = lp0.GetX()+(c1.rowOnLevel+1)*dWidth/float(countsInFraph[c1.m_level]+1);
      int y = lp4.GetY()+(lp4.GetY()<lp3.GetY() ? 1 : -1)*((c2.m_level+1)*dHeight/float(Maxlevel+2));

      lp1.SetX(x);
      lp2.SetX(x); lp2.SetY(y);
      lp3.SetY(y);

      link.GetExtraPoints().SetRefItem(1,lp1);
      link.GetExtraPoints().SetRefItem(2,lp2);
      link.GetExtraPoints().SetRefItem(3,lp3); // may be it is not needded
    }
  }
}


int getInc(short kind)
{
  if(kind==TAG_CLASS)
    return 0;
  if(kind==TAG_STRUCT)
    return TagFile.classes.m_children[TAG_CLASS].size();
  if(kind==TAG_INTERFACE)
    return TagFile.classes.m_children[TAG_CLASS].size() + TagFile.classes.m_children[TAG_STRUCT].size();
  ASSERT(false);
  return 0;
}


void addClasses(short kind)
{
  int count = TagFile.classes.m_children[kind].size();
  for(int i=0;i<count;i++)
  {
    ClassItem& item = TagFile.classes.m_children[kind][i];
    GraphClass cl;
    for(int c=0;c<item.m_ichildren.size();c++)
    {
      cl.m_ichildren.push_back(item.m_ichildren[c].index+getInc(item.m_ichildren[c].kind));
    }
    for(int p=0;p<item.m_iparents.size();p++)
    {
      cl.m_iparents.push_back(item.m_iparents[p].index+getInc(item.m_iparents[p].kind)); 
    }
    cl.m_name = item.m_name;
    classes.push_back(cl);                                    
  }
}

void TagsListDP::doit(int type)
{
  CWaitCursor wait;

  // clean
  {
    for(int qqq=0;qqq<100;qqq++)
      countsInFraph[qqq] = counts[qqq] = 0;
    maxI = -1;
    Maxlevel = -1;
    classes.clear();
    links.clear();
    m_AddFlow.GetNodes().Clear();
  }
  
  // make collection of classes
  if(type==0)
  { 
    addClasses(TAG_CLASS);
    addClasses(TAG_STRUCT);
    addClasses(TAG_INTERFACE);
  }

  // make collection of files
  else if(type==1)
  {         
    int count = TagFile.classes.m_children[TAG_FILE].size();
    classes.resize(count);
    for(int fileFrom=0;fileFrom<count;fileFrom++)
    {
      ClassItem& item = TagFile.classes.m_children[TAG_FILE][fileFrom];
      classes[fileFrom].m_name = item.m_name;
      for(int p=0;p<item.m_ichildren.size();p++)
      {
        ASSERT(item.m_ichildren[p].kind==TAG_INCLUDE);
        ClassItem& itemInclude = TagFile.classes.m_children[TAG_INCLUDE][item.m_ichildren[p].index];
        CString incName = itemInclude.m_name;
        incName.MakeLower();
        // temp realization
        for(int fileTo=0;fileTo<TagFile.m_files.size();fileTo++)
        {
          CString fileName = TagFile.m_files[fileTo].m_name.c_str();
          fileName.MakeLower();
          if(fileName.Find(incName)>=0 && fileFrom!=fileTo)
          {
            classes[fileTo].m_ichildren.push_back(fileFrom);
            classes[fileFrom].m_iparents.push_back(fileTo); 
            break;
          }
        }
      }
    }
  }

  // make project collection
  else if(type==2)
  {         
    int count = Option.projects.size();
    classes.resize(count);
    for(int fileFrom=0;fileFrom<count;fileFrom++)
    {
      Project& item = Option.projects[fileFrom];
      classes[fileFrom].m_name = item.m_name.c_str();
      for(int p=0;p<item.dependsProjects.size();p++)
      {
        string prjName = item.dependsProjects[p];
        // temp realization
        for(int fileTo=0;fileTo<Option.projects.size();fileTo++)
        {
          string depName = Option.projects[fileTo].m_name;
          if(depName==prjName)
          {
  //          classes[fileTo].m_ichildren.push_back(fileFrom);
    //        classes[fileFrom].m_iparents.push_back(fileTo); 
            classes[fileTo].m_iparents.push_back(fileFrom);
            classes[fileFrom].m_ichildren.push_back(fileTo); 
            break;
          }
        }
      }
    }
  }


  // prepare recursiveMask
  recursiveMask = new bool[classes.size()];
  
  // find levels of classes
  {
    for(int i=0;i<classes.size();i++)
    {
      if(classes[i].m_iparents.size()==0)
        do_update(i);
    }
  }

  // clear recursiveMask
  delete recursiveMask; recursiveMask = NULL;

  // find statistic
  {
    for(int i=0;i<classes.size();i++)
      classes[i].rowOnLevel = countsInFraph[classes[i].m_level]++;
  }

  draw(m_AddFlow.GetNodes());
}

/////////////////////////////////////////////////////////////////////////////
// TagsListDP message handlers
void TagsListDP::OnSave()
{
  CFileDialog dlg(FALSE,"*.emf","tags.emf",OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT|OFN_EXPLORER,"EMF File (*.emf)|*.emf|WMF File (*.wmf)|*.wmf||");
  if(dlg.DoModal()==IDOK)
    m_AddFlow.SaveImage(0,dlg.GetFileExt()=="wmf" ? 0 : 1,dlg.GetPathName());
}

void TagsListDP::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
  m_AddFlow.SetXZoom(m_zoom.GetPos());
  m_AddFlow.SetYZoom(m_zoom.GetPos());
}
           
void TagsListDP::OnClickAddflowctrl2() 
{
  CNode node = m_AddFlow.GetPointedNode();
  if(node!=NULL)
  {
    if(m_GrType!=2) // can't find projects now
    {
      CString name = node.GetToolTip();
      m_processFocus = false;
      CCommands::Get()->selectTagByName(name);
      m_processFocus = true;
    }

  // Reset DrawColor of all items
    VARIANT vaIdx;
    VariantInit(&vaIdx);
    vaIdx.vt = VT_I4;

    // Reset DrawColor of all items
	  CNodes nodes = m_AddFlow.GetNodes(); 
    {
      for (int i = 1; i <= nodes.GetCount(); i++)
      {
        vaIdx.lVal = i;
        CNode node = nodes.Item(vaIdx);
        node.SetDrawColor(RGB(0, 0, 0));
        CLinks links = node.GetOutLinks();
        for (int k = 1; k <= links.GetCount(); k++)
        {
          vaIdx.lVal = k;
          links.Item(vaIdx).SetDrawColor(RGB(0, 0, 0));
        }
      }
    }

    // color node
    node.SetDrawColor(RGB(0, 0, 255));
    // Color In links
    {
      CLinks links = node.GetInLinks();
      for (int k = 1; k <= links.GetCount(); k++)
      {
        vaIdx.lVal = k;
        CLink link = links.Item(vaIdx);
        if (!link.GetRigid())
        {
          link.SetDrawColor(RGB(255, 0, 0));
          link.GetOrg().SetDrawColor(RGB(255, 0, 0));
        }
      }
    }


    // Color Out links
    {
      CLinks links = node.GetOutLinks();
      for (int k = 1; k <= links.GetCount(); k++)
      {
        vaIdx.lVal = k;
        CLink link = links.Item(vaIdx);
        if (!link.GetRigid())
        {
          link.SetDrawColor(RGB(0, 255, 0));
          link.GetDst().SetDrawColor(RGB(0, 255, 0));
        }
      }
    }
  }
}
