/*@ NestorThreads.cpp                         Version from  19.04.99 17:47  */
// COPYRIGHT (C) 1999-2000 by Melnikov Mike.

#include "stdafx.h"

#include "process.h"
#include "NestorThreads.h"


//=================  Class Thread
unsigned int __stdcall Thread::Func(void* lpThreadParam)
{
  try
  {
    ((Thread*)lpThreadParam)->run();

    if( ((Thread*)lpThreadParam)->destroyOnExit )
      delete (Thread*)lpThreadParam;
    return 0;
  }
  catch( ... )
  {
    return 1;
  }
}

Thread::Thread(bool createSuspended ,int nPriority,bool autoDestroy) : externalHandle(NULL),destroyOnExit(autoDestroy)
{
  externalHandle = (HANDLE)_beginthreadex(NULL,0,Func,this,createSuspended ? CREATE_SUSPENDED : 0,(unsigned int*)&threadID);
  ASSERT(externalHandle);
  SetThreadPriority(externalHandle,nPriority);
}

Thread::~Thread()
{
  CloseHandle(externalHandle);
}

void Thread::resume()
{
  ResumeThread(externalHandle);
}

void Thread::suspend()
{
  SuspendThread(externalHandle);
}

void Thread::wait(DWORD milliseconds)
{
  WaitForSingleObject(externalHandle,milliseconds);
}

void Thread::terminate(DWORD exitCode)
{
  TerminateThread(externalHandle,exitCode);
}

void Thread::exit(DWORD exitCode)
{
  ExitThread(exitCode);
}

void Thread::setPriorityLevel(int new_priority_level)
{
  SetThreadPriority(externalHandle,new_priority_level);
}


//=================  Class Timer
Timer::Timer() : Thread(true,THREAD_PRIORITY_ABOVE_NORMAL),
                 call_me_please(NULL),data_for_call(NULL),
                 ce_tou(false),started(false),wait_for(0),auto_reset(true)
{
  resume();
}

void Timer::start(float seconds,Call callback,void* user_data)
{
  wait_for = int(1000*seconds);
  call_me_please = callback;
  data_for_call  = user_data;
  started = true;
  resume();
}

void Timer::reset()
{
  wait_for = 0;
  call_me_please = 0;
  data_for_call  = 0;
  started = false;
  resume();
}

void Timer::done()
{
  if( !ce_tou )
  {
    ce_tou = true;
    resume();
    wait();
  }
}

void Timer::run()
{
  while( !ce_tou )
  {
    if( !started ) suspend();
    if( !auto_reset ) suspend();
    if( started )
    {
      Sleep(wait_for);
      if( call_me_please )
        (*call_me_please)(data_for_call);
      if( !auto_reset ) reset();
    }
  }
}
