#ifndef _CPPEX_H
#define _CPPEX_H

#include <string>
#include <vector>
#include <assert.h>
using namespace std;

struct Tag
{
  Tag() {}
   virtual const char* getName() {/*assert(false);*/return "";}
  virtual void print();
  virtual Tag* insert(Tag* e) {assert(false);return 0;}
};

char* OutPutYYName(int s);
struct TagChar : Tag
{
  TagChar(int as) : s(as) {}

  int    s;
  virtual const char* getName() {return OutPutYYName(s);}
};

struct TagStr : Tag,string
{
  TagStr(const char* s) : string(s) {}
  virtual const char* getName() {return c_str();}
};

struct Tag2 : Tag
{
  Tag2(Tag* a_tag1,Tag* a_tag2)
    : tag1(a_tag1),tag2(a_tag2){} 

  Tag*   tag1;
  Tag*   tag2;
  virtual void print();
};

struct TagVector : Tag, vector<Tag*>
{
  explicit TagVector(Tag* e) {insert(e);}
  explicit TagVector(Tag* e1,Tag* e2,Tag* e3) {insert(e1);insert(e2);insert(e3);}
  Tag* insert(Tag* e) {if(e)push_back(e);return this;}
  virtual void print() 
  {
    for(int i=0;i<vector<Tag*>::size();i++) 
      (vector<Tag*>::operator[](i))->print();
  }
};

struct function_definition : Tag
{
  function_definition(Tag* a_decl_specifier_seq,Tag* a_ctor_initializer,Tag* a_declarator,Tag* a_function_body)
    : decl_specifier_seq(a_decl_specifier_seq),ctor_initializer(a_ctor_initializer),declarator(a_declarator),function_body(a_function_body){}

  Tag* decl_specifier_seq;
  Tag* ctor_initializer;
  Tag* declarator;
  Tag* function_body;
  virtual void print();
};

struct class_head : Tag
{
  class_head(Tag* a_class_key,Tag* a_nested_name_specifier,Tag* a_identifier,Tag* a_base_clause)
    : class_key(a_class_key),nested_name_specifier(a_nested_name_specifier),m_identifier(a_identifier),base_clause(a_base_clause){}
  
  Tag* class_key;
  Tag* nested_name_specifier;
  Tag* m_identifier;
  Tag* base_clause;
  virtual void print();
};

struct template_declaration : Tag
{
  template_declaration(Tag* a_export,Tag* a_template_parameter_list,Tag* a_declaration)
    : export(a_export),template_parameter_list(a_template_parameter_list),declaration(a_declaration) {}
 
  Tag* export;
  Tag* template_parameter_list;
  Tag* declaration;
  virtual void print();
};

struct class_specifier : Tag
{
  class_specifier(Tag* a_class_head,Tag* a_member_specification)
    : class_head(a_class_head),member_specification(a_member_specification){}

  Tag* class_head;
  Tag* member_specification;
  virtual void print();
};

struct simple_declaration : Tag
{
  simple_declaration(Tag* a_decl_specifier_seq,Tag* a_init_declarator_list)
    : decl_specifier_seq(a_decl_specifier_seq),init_declarator_list(a_init_declarator_list){}

  Tag* decl_specifier_seq;
  Tag* init_declarator_list;
  virtual void print();
};

struct cast_expression : Tag
{
  cast_expression(Tag* a_type_id,Tag* a_unary_expression)
    : unary_expression(a_unary_expression), type_id(a_type_id){} 

  Tag*   unary_expression;
  Tag*   type_id;
  virtual void print();
};
struct pm_expression : Tag
{
  pm_expression(Tag* a_tag1,int aop,Tag* a_tag2)
    : tag1(a_tag1),op(aop),tag2(a_tag2){} 

  Tag*   tag1;
  int    op;
  Tag*   tag2;
  virtual void print();
};

struct parameter_declaration : Tag
{
  parameter_declaration(Tag* a_decl_specifier_seq,Tag* a_declarator,Tag* a_assignment_expression=0)
    : decl_specifier_seq(a_decl_specifier_seq),declarator(a_declarator),assignment_expression(a_assignment_expression){}
  
  Tag* decl_specifier_seq;
  Tag* declarator;
  Tag* assignment_expression;
  virtual void print();
};

struct parameter_declaration_clause : Tag
{
  parameter_declaration_clause(Tag* a_parameter_declaration_list,Tag* a_OP_SOME_POINTS)
    : parameter_declaration_list(a_parameter_declaration_list),op_some_points(a_OP_SOME_POINTS){}

  Tag* parameter_declaration_list;
  Tag* op_some_points;
  virtual void print();
};

struct direct_declarator  : Tag
{
  direct_declarator(Tag* a_direct_declarator,Tag* a_constant_expression)
    :  m_direct_declarator(a_direct_declarator),parameter_declaration_clause(0),cv_qualifier_seq(0),exception_specification(0),constant_expression(a_constant_expression){}
  direct_declarator(Tag* a_direct_declarator,Tag* a_parameter_declaration_clause,Tag* a_cv_qualifier_seq,Tag* a_exception_specification)
    :  m_direct_declarator(a_direct_declarator),parameter_declaration_clause(a_parameter_declaration_clause),cv_qualifier_seq(a_cv_qualifier_seq),exception_specification(a_exception_specification),constant_expression(0){}
  Tag* m_direct_declarator;
  Tag* parameter_declaration_clause;
  Tag* cv_qualifier_seq;
  Tag* exception_specification;
  Tag* constant_expression;
  virtual void print();
};

struct member_declaration : Tag
{
  member_declaration(Tag* a_decl_specifier_seq,Tag* a_declarator)
    : decl_specifier_seq(a_decl_specifier_seq),m_declarator(a_declarator){}
  Tag* decl_specifier_seq;
  Tag* m_declarator;
  virtual void print();
};

struct member_specification : Tag
{
  member_specification(Tag* a_member_declaration,Tag* a_member_specification)
    :member_declaration(a_member_declaration),m_member_specification(a_member_specification),access_specifier(0){}
  member_specification(Tag* a_access_specifier,Tag* a_member_specification,int )
    :member_declaration(0),m_member_specification(a_member_specification),access_specifier(a_access_specifier){}
  Tag* m_member_specification;
  Tag* member_declaration;
  Tag* access_specifier;
  virtual void print();
};

struct selection_statement : Tag
{
  selection_statement(int a_type,Tag* a_condition,Tag* a_statement,Tag* a_statementElse=0)
    :type(a_type),condition(a_condition),statement(a_statement),statementElse(a_statementElse){}
  
  int  type;
  Tag* condition;
  Tag* statement;
  Tag* statementElse;
};

struct ptr_operator : Tag
{
  ptr_operator(int a_type,Tag* a_cv_qualifier_seq,Tag* a_nested_name_specifier)
    : type(a_type),cv_qualifier_seq(a_cv_qualifier_seq),nested_name_specifier(a_nested_name_specifier){}
  
  int type;
  Tag* cv_qualifier_seq;
  Tag* nested_name_specifier;
};

struct postfix_expression : Tag
{
  postfix_expression(Tag* e1,Tag* e2,Tag* e3,Tag* e4,Tag* e5)
    : t1(e1),t2(e2),t3(e3),t4(e4),t5(e5){}
  
  Tag* t1,*t2,*t3,*t4,*t5;
};

struct conditional_expression : TagVector
{
  explicit conditional_expression(Tag* e1,Tag* e2,Tag* e3) 
    : TagVector(e1,e2,e3) {}
};

struct unary_expression : Tag2
{
   unary_expression(Tag* a_tag1,Tag* a_tag2)
     : Tag2(a_tag1,a_tag2) {}
};

#endif