%{
/****************************************************************************
lexer.l
****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include "parser.h"

%}

// include file
%include {
// forward references
#include <vector>

using namespace std;
}

// lexical analyser name
%name Lexer


// class definition
{
public: 
  Lexer(const char* );
  ~Lexer();

protected:
  virtual int  yygetchar();

// Attributes
protected:
  vector<FILE*>      m_files;

  void openFile(const char* str);
  void closeFile();
  
  string*          id();
  string*          phrase();

  vector<string*> strings;

}

// constructor
{
  assert(false);
}

// macros
id        ([a-zA-Z_][a-zA-Z0-9_]*)
//
simbols   (a-zA-Z_0-9?!\\,.:\xc0-\xff)
// string - no spaces , phrase - with spaces wpuld be in ""
phrase    ([a-zA-Z_0-9-/~!@#$%^&()!?\\,.: ]*)
string    ([a-zA-Z_0-9-/~!@#$%^&()!?\\,.:]*)

%%

%{
// extract yylval for use later on in actions
YYSTYPE& yylval = *(YYSTYPE*)yyparserptr->yylvalptr;
%}

// keywords

"SOURCE"       { return SOURCE; }

// scene.command

// operators
"="             { return '=';    }
"/I"[ ]*        { return INCL;}

// white space
[ \t]            { /* do nothing*/}
\n              { return '\n'; }

"\""{phrase}"\"" { yylval.string = phrase(); return PHRASE1; }
{string} { yylval.string = id(); return STRING; }

.             { }//assert("invalid character"==0); }

%%

/////////////////////////////////////////////////////////////////////////////
// Lexer commands

Lexer::Lexer(const char* str)
{
  yytables();
  openFile(str);
}

Lexer::~Lexer()
{
  for(int i=0;i<strings.size();i++)
    delete strings[i];
}

/////////////////////////////////////////////////////////////////////////////
// Lexer attribute commands
void Lexer::openFile(const char* str)
{
  FILE* file = fopen(str,"r");
  if(file)
    m_files.push_back(file);
}
void Lexer::closeFile()
{
  if(m_files.size())
  {
    fclose(*m_files.rbegin());
    m_files.erase(m_files.end()-1);
  }
}

string* Lexer::phrase()
{
  string* s = id();
  assert(*s->begin()=='\"');
  assert(*(s->end()-1)=='\"');
  s->erase(s->end()-1);
  s->erase(s->begin());
  return s;
}

string* Lexer::id()
{
  string* p = new string(yytext);
  strings.push_back(p);
  return p;
}


//#define TRACE_AUTOMAT_PARSER 

int Lexer::yygetchar()
{
  if(!m_files.size())
    return EOF;
  int c = getc(*m_files.rbegin());
#ifdef TRACE_AUTOMAT_PARSER
  char str[2];
  str[0] = c;
  str[1] = 0;
  OutputDebugString(str);
#endif
  switch(c)
  {
    case EOF:
      closeFile(); // retrun \n
    case '\n' :
    case '\r' :
    case ';'  :
     return '\n';
  }
  return c;
}
