%{
/****************************************************************************
parser.y
****************************************************************************/

#include <math.h>
#include "..\tags\Options.h"
%}

//include file
%include {
// forward references
#include <string>
using namespace std;

#pragma warning ( disable : 4786 ) //identifier was truncated to 'number' characters in the debug information

}



// attribute type
%union {
//  D_Operator*      doperator; // operators
  string*         string; // id
}

// nonterminals
%type <doperator> expr

// tokens
%right '='

%token <string>      ID
%token <string>      STRING
%token <string>      PHRASE1

// keywords
%token SOURCE
%token INCL

// include file
%include {
#include "lexer.h"
#include "..\tags\entry.h"
}

// parser name
%name Parser

// class definition
{
// Attributes
protected:
  Lexer m_lexer;       // the lexical analyser
  
// Operations
public:
  Parser(Project* project) : m_lexer(project->fileName.c_str()),m_project(project)
                        {yytables();create();}
  
  // attribute commands
  void              newProject(string* project,string* file);
  void              newInclude(string* include);
  void              newSource(string* file);
  
private:  
  int               create();
  bool              normalize(string*& file);
  
  Project*          m_project;
}

// constructor
{
  assert(false);
}

%%
lines
  : lines EMPTY
  | lines line
  | /* empty */
  ;

line
  : expr '\n'         { /*addOperator($1);*/}
  | error '\n'        { yyerrok(); }
  ;

EMPTY
  : STRING {/* unknown*/}
  | INCL  PHRASE1 { newInclude($2);}
  | SOURCE '=' STRING { newSource($3);}
  | SOURCE '=' PHRASE1 { newSource($3);}
  | '\n'              {}
  ;

expr
  : ID '=' expr         { /*$$ = assign($1, $3);*/}
//  | expr '+'  expr      { $$ = new D_Add($1,$3);}
//  | expr '-'  expr      { $$ = new D_Substruct($1,$3);}
//  | expr '*'  expr      { $$ = new D_Multiplication($1,$3);}
//  | expr '/'  expr      { $$ = new D_Division($1,$3);}
//  | expr OR       expr  { $$ = new D_Or($1,$3);}
//  | expr AND      expr  { $$ = new D_And($1,$3);}
//  | expr EQUAL    expr  { $$ = new D_IsEqual($1,$3);}
//  | expr NOTEQUAL expr  { $$ = new D_IsNotEqual($1,$3);}
//  | expr LESS     expr  { $$ = new D_IsLess($1,$3);}
//  | expr GREATER  expr  { $$ = new D_IsGreater($1,$3);}
//  | '(' expr ')'        { $$ = $2;}
//  | CONSTANT            { $$ = $1;}
//  | FUNC ARG ')'        { $$ = callFunction($1);}
//  | DOON expr ',' expr ')' {$$ = makeDoAfter($2,$4);}
//  | ID                  { $$ = new D_Variable(NameLinkID(*$1)); }
  ;


%%

/////////////////////////////////////////////////////////////////////////////
// Parser commands

int Parser::create()
{
  if (!yycreate(&m_lexer))
    return 0;
  if (!m_lexer.yycreate(this))
    return 0;
  return 1; // success
}

/////////////////////////////////////////////////////////////////////////////
// Parser attribute commands

bool Parser::normalize(string*& file)
{
  if ((*file)[0] == '.')
    *file = m_project->path + string("\\") + *file;
  CFileStatus fileStatus;
  if(CFile::GetStatus(file->c_str(), fileStatus))
  {
    *file = fileStatus.m_szFullName;
    return true;
  }
  return false;
}

void Parser::newProject(string* project,string* file)
{
  assert(false);
}
void Parser::newInclude(string* include)
{
  if(normalize(include))
   m_project->additionalDirs.push_back(*include);
}
void Parser::newSource(string* file)
{
  if(normalize(file))
    if(getExtensionLanguage(file->c_str()) != LANG_IGNORE)
      m_project->files.push_back(*file);
}
