//----------------------------------------------------------------------------
// based on Borland WinSys Library
// Copyright (c) 1994, 1998 by Borland International, All Rights Reserved
//
//
// Modified by Mike Melnikov
//
// General Registry access & registration implementation
//----------------------------------------------------------------------------
#include "stdafx.h"

#include "registry.h"
#include "assert.h"
#include "RegularExpressions.h"

//
// Predefined constant registry keys
//
RegKey  RegKey::ClassesRoot(HKEY_CLASSES_ROOT, true, "HKEY_CLASSES_ROOT");
RegKey  RegKey::ClassesRootClsid(ClassesRoot, "CLSID", KEY_ALL_ACCESS, NoCreate);
RegKey& RegKey::GetClassesRoot() {return ClassesRoot;}
RegKey& RegKey::GetClassesRootClsid() {return ClassesRootClsid;}
RegKey& RegKey::GetCurrentUser()
{
  static RegKey CurrentUser(HKEY_CURRENT_USER, true, "HKEY_CURRENT_USER");
  return CurrentUser;
}
RegKey& RegKey::GetLocalMachine()
{
  static RegKey LocalMachine(HKEY_LOCAL_MACHINE, true, "HKEY_LOCAL_MACHINE");
  return LocalMachine;
}
RegKey& RegKey::GetUsers()
{
  static RegKey Users(HKEY_USERS, true, "HKEY_USERS");
  return Users;
}
RegKey& RegKey::GetCurrentConfig()
{
  static RegKey CurrentConfig(HKEY_CURRENT_CONFIG, true, "HKEY_CURRENT_CONFIG");
  return CurrentConfig;
}
RegKey& RegKey::GetDynData()
{
  static RegKey DynData(HKEY_DYN_DATA, true, "HKEY_DYN_DATA");
  return DynData;
}
RegKey& RegKey::GetPerformanceData()
{
  static RegKey PerformanceData(HKEY_PERFORMANCE_DATA, true, "HKEY_PERFORMANCE_DATA");
  return PerformanceData;
}

//
// Initialize the object with the passed-in data.
//
RegKey::RegKey(THandle baseKey, const _TCHAR far* keyName, REGSAM samDesired, TCreateOK createOK)
:
  Name(strnewdup(keyName)),
  Key(0)
{
  if (createOK == CreateOK) {
    unsigned long disposition;
    ::RegCreateKeyEx(baseKey, Name, 0, ""/*class*/, REG_OPTION_NON_VOLATILE,
                     samDesired, 0/*SecurityAttributes*/, &Key,
                     &disposition);
  }
  else
    ::RegOpenKeyEx(baseKey, Name, 0, samDesired, &Key);

  QueryInfo(0, 0, &SubkeyCount, 0, 0, &ValueCount, 0, 0, 0, 0);

  ShouldClose = true;

}

//
// Initialize object with the key pointed to by the iterator.
//
RegKey::RegKey(const RegKeyIterator& iter, REGSAM samDesired)
:
  Key(0),
  Name(new _TCHAR[_MAX_PATH+1])
{
  iter.BaseKey().EnumKey(iter.Current(), Name, _MAX_PATH+1);
  ::RegOpenKeyEx(iter.BaseKey(), Name, 0, samDesired, &Key);

  ShouldClose = true;
  QueryInfo(0, 0, &SubkeyCount, 0, 0, &ValueCount, 0, 0, 0, 0);
}

//
// Construct a RegKey object around an existing key. Commonly used for
// wrapping the preexisting pseudo-keys.
//
RegKey::RegKey(THandle aliasKey, bool shouldClose, const _TCHAR far* keyName)
:
  Key(aliasKey),
  Name(strnewdup(keyName))
{
  ShouldClose = shouldClose;

  __try
  {
    long err = QueryInfo(0, 0, &SubkeyCount, 0, 0, &ValueCount, 0, 0, 0, 0);
    if (err != S_OK && err != ERROR_INSUFFICIENT_BUFFER)
      Key = 0;
  }
  // Some key handles are unsupported & sometimes the OS crashes, & doesn't
  // just return an error. Catch it here & zero out this key.
  //
  __except ((GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION) ?
              EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH) {
    Key = 0;
  }
}

//
// Destructor closes the key if necessary and deletes any previously
// allocated memory by the object.
//
RegKey::~RegKey()
{
  if (Key && ShouldClose)
    ::RegCloseKey(Key);
  delete[] Name;
}
                         
//
// Wrapper for RegQueryInfoKey API.
//
long
RegKey::QueryInfo(_TCHAR far* class_, unsigned long* classSize,
                   unsigned long* subkeyCount,  unsigned long* maxSubkeySize,
                   unsigned long* maxClassSize, unsigned long* valueCount,
                   unsigned long* maxValueName, unsigned long* maxValueData,
                   unsigned long* secDescSize, PFILETIME lastWriteTime)
{
  _TCHAR      db[256];
  unsigned long    d1 = sizeof db;
  unsigned long    d2 = 0;
  unsigned long    d3 = 0;
  unsigned long    d4 = 0;
  unsigned long    d5 = 0;
  unsigned long    d6 = 0;
  unsigned long    d7 = 0;
  unsigned long    d8 = 0;
  FILETIME  ft;

  // NT 3.50 has trouble with 0 ptrs for some of these
  //
  return ::RegQueryInfoKey(Key, class_ ? class_ : db,
                           classSize ? classSize : &d1, 0,
                           subkeyCount ? subkeyCount : &d2,
                           maxSubkeySize ? maxSubkeySize : &d3,
                           maxClassSize ? maxClassSize : &d4,
                           valueCount ? valueCount : &d5,
                           maxValueName ? maxValueName : &d6,
                           maxValueData ? maxValueData : &d7,
                           secDescSize ? secDescSize : &d8,
                           lastWriteTime ? lastWriteTime : &ft);
}

//
// Completely nuke a child key, including any of its subkeys. RegDeleteKey
// fails if a key has subkeys, so must tail-recurse to clean them up first.
//
long
RegKey::NukeKey(const _TCHAR far* subKeyName)
{
  {
    RegKey subKey(*this, subKeyName, KEY_ALL_ACCESS, RegKey::NoCreate);

    // Don't increment since deleting & all subkeys will slide down to 0
    //
    for (RegKeyIterator i(subKey); i; ) {
      RegKey subSubKey(i);
      if (!subSubKey)
        break;     // All done, ran out before iterator knew, since deleting
      long err = subKey.NukeKey(subSubKey.GetName());
      if (err)
        return err;
    }
  }
  return DeleteKey(subKeyName);
}


void RegKey::WriteRegistry(FILE* fp,CString parent)
{
  fprintf(fp,"[%s\\%s]\n",parent,GetName());
  for( RegValueIterator it3(*this);it3;it3++)
  { 
    RegValue val(it3);
    CString name = val.GetName();
    if(name.IsEmpty())
      name = '@';
    else
      name = "\"" + name + "\"";
    switch(val.GetDataType())
    {
     case REG_SZ:
       {
         string str = (const char*)val;
         replace(str,"\\","\\\\");
         replace(str,"\"","\\\"");
         fprintf(fp,"%s=\"%s\"\n",name,str.c_str());
       }
       break;
     case REG_DWORD:
       fprintf(fp,"%s=dword:%08x\n",name,(unsigned long)val);
       break;
     case REG_BINARY:
       fprintf(fp,"%s=hex:",name);
       {
         const unsigned char* hex = val.GetData();
         int lines = 0;
         int c = strlen("=hex:") + strlen(name);
         for(int i=0; i<val.GetDataSize();i++)
         {
           fprintf(fp,"%02x",val.GetData()[i]);
           c+=2;
           if(i!=val.GetDataSize()-1)
           {
             fputc(',',fp);
             c++;
             if(c>=76)
             {
               fputs("\\\n  ",fp);
               c = 2;
               lines++;
             }
           }
         }
       }
       fprintf(fp,"\n");
       break;
     default:
       assert(false);
    }
  }
  fprintf(fp,"\n");
  for( RegKeyIterator it2(*this);it2;it2++)
  { 
    RegKey key2(it2);
    key2.WriteRegistry(fp,parent+"\\"+GetName());
  }
}


///////////////////////////////////////////////////////////////
#ifdef _MY_REG_MODE

void RegKey::Write(FILE* fp,int lev)
{
  CString nInitialScopeSpaces( ' ', lev*2 + 1);
  CString nInitialSpaces( ' ', (lev+1)*2 + 1);
  fprintf(fp,"%s[%s]\n%s{\n",nInitialScopeSpaces,GetName(),nInitialScopeSpaces);
  for( RegKeyIterator it2(*this);it2;it2++)
  { 
    RegKey key(it2);
    key.Write(fp,lev+1);
  }
  for( RegValueIterator it3(*this);it3;it3++)
  { 
    RegValue val(it3);
    if(val.GetDataType()==1)
      fprintf(fp,"%s%s = %s\n",nInitialSpaces,val.GetName(),(const char*)val);
    else if(val.GetDataType()==4)
      fprintf(fp,"%s%s = %d\n",nInitialSpaces,val.GetName(),(unsigned long)val);
  }
  fprintf(fp,"%s}\n",nInitialScopeSpaces);
}

const char
  TopicBegin   = '[',
  TopicEnd     = ']',
  ScopeEnd     = '}',
  KeySeparator = '=',
  CommentChar  = ';',
  EndLine      = '\n';

static const char WhiteSpaces[] = " \t\n\r";
static const char OpenScope[] = " \t\n\r{";
static const char CloseScope[] = " \t\n\r}";

static int skip_to(FILE *fp,const char* what)
{
  int c;
  do
    c = getc(fp);
  while (c != EOF && strchr(what,c) );
  ungetc(c,fp);
  return c;
}

static int readString( FILE *fp,CString& s,int end )
{
  int c;
  s.Empty();
  do
  {
    c = getc(fp);
    if(c == EOF || c==end)
      break;
    s += c;
  }
  while (true);
  s.TrimLeft(' ');s.TrimRight(' ');
  return c;
}

void RegKey::Read(FILE* fp,int lev)
{
  CString str,str1;
  int c = skip_to(fp,WhiteSpaces);
  ASSERT(getc(fp)==TopicBegin);
  readString(fp,str,TopicEnd);
  RegKey newKey(*this,str);
  skip_to(fp,OpenScope);
  do
  {
    int c1 = getc(fp);
    ungetc(c1,fp);
    if(c1==TopicBegin)
    { 
      newKey.Read(fp,lev+1);
    }
    else
    {
      readString(fp,str,KeySeparator);
      RegValue value(newKey,str);
      readString(fp,str1,EndLine);
      value.Set(str1);
    }
    c = skip_to(fp,WhiteSpaces);
  }while(c!=ScopeEnd);
  skip_to(fp,CloseScope);
}

void RegKey::Write(const _TCHAR far* fileName)
{
  FILE *fp = fopen(fileName,"w");
  int lev = 0;
  Write(fp,lev);
  fclose(fp);
}

void RegKey::Read(const _TCHAR far* fileName)
{
  FILE *fp = fopen(fileName,"r");
  int lev = 0;
  Read(fp,lev);
  fclose(fp);
}
#endif _MY_REG_MODE
///////////////////////////////////////////////////////////////

void RegKey::WriteRegistry(const _TCHAR far* fileName)
{
  FILE *fp = fopen(fileName,"w");
  int lev = 0;
  fprintf(fp,"REGEDIT4\n\n");
  WriteRegistry(fp,RegKey::GetCurrentUser().GetName());
  fclose(fp);
}

//
// Create a value object named 'name' associated with the the current key.
//
RegValue::RegValue(const RegKey& key, const _TCHAR far* name) : Key(key), Name(name), AName(0), Data(0)
{
  DataSize = 0;
  Key.QueryValue(Name, &DataType, 0, &DataSize); // Get type & size only
}

//
// Create a value object pointed to by the current position of the iterator.
//
RegValue::RegValue(const RegValueIterator& iter) : Key(iter.BaseKey()), Data(0)
{
  unsigned long nameSize = _MAX_PATH; //0;  
  Name = AName = new _TCHAR[(int)nameSize];
  DataSize = 0;
  Key.EnumValue(iter.Current(), AName, nameSize, &DataType, 0, &DataSize);
}

//
// Destructor deletes any previously allocated memory.
//
RegValue::~RegValue()
{
  delete[] AName;
  delete[] Data;
}

//
// Retrieve the data for the value.
//
void
RegValue::RetrieveOnDemand() const
{
  if (!Data) 
  {
#if defined(BI_NO_MUTABLE)
    CONST_CAST(RegValue*,this)->Data = new unsigned char[DataSize];
#else
    Data = new unsigned char[(int)DataSize];
#endif
    unsigned long ds = DataSize;
    Key.QueryValue(Name, 0, Data, &ds);  // DataSize shouldn't change...
  }
}

//
// Set the data for the value.
//
long
RegValue::Set(unsigned long type, unsigned char* data, unsigned long dataSize)
{
  long ret = Key.SetValue(Name, type, data, dataSize);
  if (ret == 0) 
  {
    DataType = type;
    if (dataSize == DataSize) 
    {
      memcpy(Data, data, (int)DataSize);  // resync w/ new data if same size
    }
    else 
    {
      DataSize = dataSize;           // Otherwise retrieve on demand later
      delete[] Data;
      Data = 0;
    }
  }
  return ret;
}

