#ifndef _string_HPP_
#define _string_HPP_
/* string.HPP            Version from 27 august 1997 */
/* Contents ------------------------------------------------------------------
**
**  string
**    null-terminating string
**
**  RegularExpression
**    Provides UNIX-style regular expression.
**    Regular expressions are basically sophisticated wild cards.
**    They provide the ability to deal with unprintable characters like
**    `beginning of line' or `end of line', specific sets or `classes'
**    of characters, strings of unknown length, multiple groups of
**    characters in a single search or replace pattern, and the
**    ability to search across lines.
**
**      ?  match 0 or 1 occurrences of the
**         previous character or expression.
**      *  maximal match 0 or more occurrences of the
**         previous character or expression.
**      +  maximal match 1 or more occurrences of the
**         previous character or expression.
**      .  match any character.
**     []  match class of characters.
**    [^]  match any character except the enclosed class.
**      ^  match beginning of line.
**      $  match end of line.
**      \  match (or replace) next character literally
**     ()  encloses a group of 1 or more expressions.
**      |  match previous OR next character or expression.
**
**----------------------------------------------------------------------------
**
** Designed and written by Alexander Gavrilov.
**
** -------------------------------------------------------------------------*/

#include <string>
using namespace std;

class ATL_NO_VTABLE RegularExpression
{
 public:

  //------------------------------------------------------------- constructors
  RegularExpression();
  RegularExpression( const string& );


  //-------------------------------------------------------------- assignments
  RegularExpression& operator=( const char* );


  //----------------------------------------------------------- get expression
  operator const char *() const;

  //------------------------------------------------------------- match string

  bool index( const char *s, int& rstart, int& rlength ) const;
    // returns True if the expression was found.
    // rstart  contains the position of the first instance of 'this' in s.
    // rlength contains length of matched substring.


  //--------------------------------------------------------------- comparison

  bool operator == ( const CString& ) const;
  bool operator != ( const CString& ) const;

  static bool  CaseSensitive;

  private:
  string      expression;           // holds the regular expression
  string      rePattern;            // holds the compiled version

  void        MakePattern();        // entry point to compile pattern
};
inline RegularExpression::operator const char *() const { return expression.c_str(); }

inline bool RegularExpression::operator != ( const CString& s ) const
  {  return !(*this == s); }

void replace(string& str,string::size_type start, string::size_type end, const char *new_str );
int  replace(string& str,const RegularExpression& from, const char *to, bool all =true);

int  replace(string& str,const string& from, const string& to, bool all =true);

void TrimRight(CString& str,const char* c);
void TrimLeft(CString& str,const char *c);

#endif /* of _STRING_HPP_ */
