/*****************************************************************************
*   $Id: entry.h,v 8.2 1999/03/27 21:08:52 darren Exp $
*
*   Copyright (c) 1998-1999, Darren Hiebert
*
*   This source code is released for free distribution under the terms of the
*   GNU General Public License.
*
*    al interface to entry.c
*****************************************************************************/
#ifndef _ENTRY_H
#define _ENTRY_H

#include <afx.h>

#include "ctags.h"
#include "KEYWORD.H"
#include "vector"
#include <string>
#include <functional>

using namespace std;

void baseFilename(const CString& file,CString& fileName,CString& filePath);

char* g_tagFolders[];

/*  Describes the type of tag being generated.
*/
enum  tagType {
    TAG_UNDEFINED = -1,
    TAG_CLASS,			    /* class name */
    TAG_STRUCT,			    /* structure name */
    TAG_FUNCTION,		    /* function definition */
    TAG_ENUM,			      /* enumeration name */
    TAG_ENUMERATOR,		  /* enumerator (enumeration value) */
    TAG_UNION,			    /* union name */
    TAG_DEFINE,         // #define
    TAG_INTERFACE,		  /* interface declaration */
    TAG_MEMBER,			    /* structure, class or interface member */
    TAG_TYPEDEF,		    /* typedef name */
    TAG_VARIABLE,		    /* variable definition */
    TAG_EXTERN_VAR,     /*  al variable declaration */
    TAG_FILE,           // file name
    TAG_INCLUDE,        // include header
    TAG_COUNT,			    /* must be last */
};

enum ParseState {_ShouldParse,_ShouldParseCommon,_DontParse,_AlreadyParsed,_MaybeParse};

struct Project
{
  string m_name;
  string fileName;
  string path;
  vector<string> m_files;
  vector<string> m_shortNames; // for "smart" includes
  vector<string> additionalDirs;
  vector<string> dependsProjects;

  void addFile(string file,ParseState state);

  static bool Normalize(const string& path,const string& file,string& testFile);

  static Project commonProject;
};


struct TagItem
{
  TagItem() : file(-1),line(-1) {}
  TagItem(WORD f,WORD l) : file(f),line(l){}

#if _MSC_VER < 1200
  friend inline bool operator<(const TagItem& x, const TagItem& y) {return x.value < y.value; }
  friend inline bool operator==(const TagItem& x, const TagItem& y) {return x.value == y.value; }
#endif

//  TagItem(WORD k,WORD i) : kind(k),index(l){}
  union
  {
    DWORD    value;
    struct
    {
      union
      {
        WORD kind;
        WORD file;
      };
      union
      {
        WORD index;
        WORD line;
      };
    };
  };
};

typedef vector<TagItem>  TagVector;

inline void operator >> (CArchive& fp,TagItem& item){fp>>item.value;}
inline void operator << (CArchive& fp,const TagItem& item){  fp<<item.value;}


struct ClassItem
{
  ClassItem(){}
  ClassItem::ClassItem(const ClassItem& item) { assign(item);}
  void operator=(const ClassItem& item) { assign(item);}

  void assign(const ClassItem& item);

  void clear();
  bool isEmpty();

  short      getLine();
  CString    getFile();
  short      countDeltaLine(short lineIn,int fileIndex);

  CString    m_name;
  short      fileIndex;
  short      m_line;

  vector<ClassItem> m_children[TAG_COUNT];

  TagVector    m_ichildren;
  TagVector    m_iparents;

  CString      classParents;        // only to fill ichildrens, iparents
  CString      parents;  // only to build info
};

typedef vector<ClassItem> VecClassItem;
typedef VecClassItem::iterator ClassItemIter;

inline bool operator<(const ClassItem& x, const ClassItem& y)
{return strcmp(x.m_name,y.m_name)<0; }

#if _MSC_VER < 1200
  inline bool operator==(const ClassItem& x, const ClassItem& y) {return &x == &y; }
#endif

void operator >> (CArchive& fp,ClassItem& item);
void operator << (CArchive& fp,const ClassItem& item);

template <class T>
void operator >> ( CArchive& fp, vector<T>& vect )
{
  short count;
  fp >> count;
  vect.resize(count);
  for(short j=0; j<count;j++)
    fp >> vect[j];
}

template <class T>
void  operator << ( CArchive& fp, const vector<T>& vect )
{
  short icount = vect.size();
  fp << icount;
  for(short j=0; j<icount;j++)
    fp << vect.at(j);
}

struct SourceFile
{
  SourceFile() : project(0),m_shouldParse(_DontParse){}
  string                   m_name;
  Project*                 project;
  ParseState               m_shouldParse;
};

inline void operator >> (CArchive& fp,SourceFile& item){CString name;fp>>name;item.m_name=name;}
inline void operator << (CArchive& fp,const SourceFile& item){  fp<<CString(item.m_name.c_str());}

struct tagFile
{
  ClassItem          classes;
  ClassItem          childrens;
  vector<SourceFile> m_files;
  int                currentFile;
  short              getFileIndex(string str);
};

void writeCtagsEntry(ClassItem& tag,tagType type );

// utils
void addSlash(CString& str);
void baseFilename(const CString& file,CString& fileName,CString& filePath);
CString contact(CString& path,LPCTSTR file);

namespace std
{
  struct less<SourceFile> : public binary_function<SourceFile, SourceFile, bool> 
  {
    bool operator()(const SourceFile& x, const SourceFile& y) const
    {
      return _stricmp(x.m_name.c_str(),y.m_name.c_str())<0;
    }
  };

  struct equal_to<SourceFile> : public binary_function<SourceFile, SourceFile, bool> 
  {
    bool operator()(const SourceFile& x, const SourceFile& y) const
    {
      return _stricmp(x.m_name.c_str(),y.m_name.c_str())==0;
    }
  };
}


template<class T>
void sortAndUnique(vector<T>& vect)
{
  sort(vect.begin(),vect.end(),less<T>());
  T* last =  unique(vect.begin(),vect.end(),equal_to<T>());
  vect.erase(last,vect.end());
}


#endif	/* _ENTRY_H */
