/*****************************************************************************
*   $Id: main.c,v 8.5 1999/06/22 02:29:38 darren Exp $
*
*   Copyright (c) 1996-1999, Darren Hiebert
*
*   Author: Darren Hiebert <darren@hiebert.com>, <darren@hiwaay.net>
*           http://darren.hiebert.com
*
*   This source code is released for free distribution under the terms of the
*   GNU General Public License. It is provided on an as-is basis and no
*   responsibility is accepted for its failure to perform as expected.
*
*   This is a reimplementation of the ctags(1) program. It is an attempt to
*   provide a fully featured ctags program which is free of the limitations
*   which most (all?) others are subject to.
*
*   This module contains top level start-up and portability functions.
// Modified by Mike Melnikov
*****************************************************************************/
#include "stdafx.h"

#include "options.h"
#include "parse.h"
#include "read.h"
#include "Entry.h"

#include <algorithm>
//#include "..\resource.h"
#include "..\utils.h"
#include "..\Registry.h"
#include "parser.h"
#include "dswparser.h"
#include "..\Commands.h" // for RefreshCommon

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

using namespace std;

extern  tagFile TagFile;

// too bad
extern CProgressCtrl* CProgress[3];
extern CString GetModuleVersion(LPTSTR mod);

void SetPos(int num,int p)
{
  if(CProgress[num])
    CProgress[num]->SetPos(p);
}

static void writeTagFile()
{
  CFile file(Option.tagFileName, CFile::modeCreate | CFile::modeWrite);
  CArchive fp(&file, CArchive::store);
  short version = 3;
  fp << version;
  fp << TagFile.classes;
  fp << TagFile.m_files;
}

static void updatePseudoTags()
{
	TRY
  {
    CFile file(Option.tagFileName, CFile::modeRead);
    CArchive fp(&file, CArchive::load);
    short version;
    fp >> version;
    fp >> TagFile.classes;
    if(version==1)
    {
      vector<CString> files;
      fp >> files;
      for(int i=0;i<files.size();i++)
      {
        SourceFile file;
        file.m_name = files[i];
        TagFile.m_files.push_back(file);
      }
    }
    else
    {
      fp >> TagFile.m_files;
    }
    
    if(version<3)
    {
      TagFile.classes.clear();
      TagFile.m_files.clear();
      AfxMessageBox("Old version of tags.dat. It will be rebuild...");
    }
  }
	CATCH(CFileException, e)
  {
    return;
  }
	END_CATCH
}


// The buffer will be modified.
void ProcessPaths(const char* buffer)
{
	char* ptr;
	char* lastPtr = const_cast<char*>(buffer);
	do
	{
		ptr = strchr(lastPtr, ';');
		if (ptr)
			*ptr++ = 0;
    Project::commonProject.additionalDirs.push_back(lastPtr);
		lastPtr = ptr;
	} while (ptr);
}

// Refresh the path list.
void RefreshCommon()
{
	// Read the registry to get the paths.
  Project::commonProject.additionalDirs.clear();
  Project::commonProject.m_files.clear();
  Project::commonProject.m_shortNames.clear();

  CString ver = GetModuleVersion("MSDEV.EXE");
  ASSERT(ver.GetLength());
  CString str;
  str.Format("Software\\Microsoft\\Devstudio\\%c.0\\Build System\\Components\\Platforms\\Win32 (x86)\\Directories",ver[0]);
  RegKey key(RegKey::GetCurrentUser(),str);
  ProcessPaths(RegValue(key,"Include Dirs"));
	ProcessPaths(RegValue(key,"Source Dirs"));
  sortAndUnique(Project::commonProject.additionalDirs);
}

void createTagsForFile(string filePath,const langType language)
{
  if (fileOpen(filePath.c_str(), language))
  {
    ASSERT(TagFile.currentFile == TagFile.getFileIndex(filePath.c_str()));

    createCTags();
    fileClose();
  }
}

void addCommonFiles(string pattern,ParseState shouldParse)
{
  CFileFind finder;
  BOOL bWorking =  finder.FindFile(pattern.c_str());
  while (bWorking)
  {
    bWorking = finder.FindNextFile();
    //  We must not recurse into the directories "." or "..".
    if(!finder.IsDots())
    {
  		if(finder.IsDirectory())
      {
        string newdir = finder.GetFilePath() + "\\*.*";
        addCommonFiles(newdir,shouldParse);
        continue;
      }
      if (getExtensionLanguage(finder.GetFileName()) != LANG_IGNORE)
      {
        Project::commonProject.addFile((const char*)finder.GetFilePath(),shouldParse);
      }
    }
  }
}

static ClassItem* findItem(ClassItem& itemF,VecClassItem& vect)
{
  if(!vect.begin())
    return NULL;
  ClassItem *found = lower_bound(vect.begin(),vect.end(),itemF);
  ASSERT(found);
  if(found!=vect.end() && !(itemF<*found))
    return found;
  return NULL;
}

void replaceLine(ClassItem* current,ClassItem* item)
{
  if(item->m_line>0)
    current->m_ichildren.push_back(TagItem(item->fileIndex,item->m_line));
  else
    current->m_iparents.push_back(TagItem(item->fileIndex,-item->m_line));
}

void splitFunctions(VecClassItem& vect)
{
  CString nowName;
  ClassItemIter current = vect.begin()-1;
  for (ClassItemIter it = vect.begin(); it!= vect.end();it++)
  {
    if(it->m_name!=nowName)
    {
      current++;
      nowName = it->m_name;
      // add first line
      replaceLine(it,it);
      it->m_line = 0;
      it->fileIndex = -1;
      // move to
      if(current!=it)
        *current = *it;
      continue;
    }
    // add to previouse
    replaceLine(current,it);
  }
  // delete tail
  vect.erase(current+1,vect.end());
}

const char seps[]   = "\\";

ClassItem itemF;

bool doFindParentChildren(ClassItemIter& itAll,vector<ClassItem>& vecC,vector<ClassItem>& vecP,short kindC,short kindP)
{
  ClassItem *found = findItem(itemF,vecP);
  if(found)
  {
    int delta1 = itAll - vecC.begin();
    found->m_ichildren.push_back(TagItem(kindC,delta1));

    int delta2 = found - vecP.begin();
    itAll->m_iparents.push_back(TagItem(kindP,delta2));
  }
  return  found!=0;
}

void findParentChildren(short kind)
{
  vector<ClassItem>& vecC = TagFile.classes.m_children[TAG_CLASS];
  vector<ClassItem>& vecS = TagFile.classes.m_children[TAG_STRUCT];
  vector<ClassItem>& vecI = TagFile.classes.m_children[TAG_INTERFACE];

  vector<ClassItem>& vec = TagFile.classes.m_children[kind];
  
  for (ClassItemIter itAll = vec.begin(); itAll!= vec.end();itAll++)
  {
    char *token = strtok((char*)(LPCTSTR)itAll->classParents,seps);
    while(token!=NULL)
    {
      itemF.m_name = token;
      
      if(!doFindParentChildren(itAll,vec,vecC,kind,TAG_CLASS))
       if(!doFindParentChildren(itAll,vec,vecS,kind,TAG_STRUCT))
            doFindParentChildren(itAll,vec,vecI,kind,TAG_INTERFACE);

      token = strtok( NULL, seps );
    }
  }
}


static void resortTags()
{
  //  
  {
    for(short kind=0;kind<TAG_COUNT;kind++)
      sort(TagFile.classes.m_children[kind].begin(), TagFile.classes.m_children[kind].end());
  }

  SetPos(2,25);

  {
    for(short kind=0;kind<TAG_COUNT;kind++)
    {
      for (ClassItemIter itAll = TagFile.childrens.m_children[kind].begin(); itAll!= TagFile.childrens.m_children[kind].end();itAll++)
      {
        ClassItem* item = &TagFile.classes;

        char *token = strtok((char*)(LPCTSTR)itAll->parents, seps );
        while(token!=NULL && item!=NULL)
        {
          itemF.m_name = token;
          if(kind==TAG_ENUMERATOR)
          {
            item = findItem(itemF,item->m_children[TAG_ENUM]);
          }
          else
          {
            ClassItem* itemNew = findItem(itemF,item->m_children[TAG_CLASS]);
            if(itemNew)
              item = itemNew;
            else
            {
              itemNew = findItem(itemF,item->m_children[TAG_STRUCT]);
              if(itemNew)
                item = itemNew;
              else
                item = findItem(itemF,item->m_children[TAG_INTERFACE]);
            }
          }

          token = strtok( NULL, seps );
        }

        if(item)
        {
          item->m_children[kind].push_back(*itAll);
          //          enum
          //     
          if(kind==TAG_CLASS || kind==TAG_STRUCT || kind==TAG_INTERFACE)
            sort(item->m_children[kind].begin(), item->m_children[kind].end());
        }
      }
    }
  }

  SetPos(2,50);

  TagFile.childrens.clear();

  //   
  {
   for(short i=0;i<TAG_COUNT;i++)
     for (ClassItemIter itParent = TagFile.classes.m_children[i].begin(); itParent!= TagFile.classes.m_children[i].end();itParent++)
       for(short kind=0;kind<TAG_COUNT;kind++)
       {
          sort(itParent->m_children[kind].begin(), itParent->m_children[kind].end());
          //      ( )      
          if(kind==TAG_FUNCTION)
            splitFunctions(itParent->m_children[kind]);
       }
  }
  // -   

  SetPos(2,75);

  splitFunctions(TagFile.classes.m_children[TAG_FUNCTION]);
  splitFunctions(TagFile.classes.m_children[TAG_DEFINE]);
  splitFunctions(TagFile.classes.m_children[TAG_VARIABLE]);
  splitFunctions(TagFile.classes.m_children[TAG_EXTERN_VAR]);


  //      - 
  {
    findParentChildren(TAG_CLASS);
    findParentChildren(TAG_STRUCT);
    findParentChildren(TAG_INTERFACE);
  }

  //     includ
  {
    short kind = TAG_INCLUDE;
    CString nowName;
    ClassItemIter current = TagFile.classes.m_children[kind].begin()-1;
    for (ClassItemIter it = TagFile.classes.m_children[kind].begin(); it!= TagFile.classes.m_children[kind].end();it++)
    {
      if(it->m_name!=nowName)
      {
        current++;
        nowName = it->m_name;
        // add first line
        replaceLine(it,it);
        it->m_line = 0;
//        it->fileIndex = 0;
        // move to
        if(current!=it)
          *current = *it;
      }
      else
      {
        // add to previouse
        replaceLine(current,it);
        current->m_iparents.push_back(it->m_iparents[0]);
      }

      WORD delta = current - TagFile.classes.m_children[kind].begin();
      WORD file =  current->m_iparents[current->m_iparents.size()-1].line; // "line" is index in massive of files
      TagFile.classes.m_children[TAG_FILE][file].m_ichildren.push_back(TagItem(kind,delta));
    }
    // delete tail
    TagFile.classes.m_children[kind].erase(current+1,TagFile.classes.m_children[kind].end());
  }
}

void Search()
{
  ASSERT(Option.tagFileName);

  // make projects vector
  Option.projects.clear();
	for (ProjectIterator pp;pp;)
	{
    Project project;
    project.m_name = pp.getName();
    project.fileName = pp.getFullName();
    CString fileName,filePath;
    baseFilename(CString(project.fileName.c_str()),fileName,filePath);
    project.path = filePath;
    Option.projects.push_back(project);
	}

  // make dependency map
  CFileFind finder;
  if(finder.FindFile(CCommands::DSWPath+"*.dsw"))
  {
    VERIFY(!finder.FindNextFile());
    DswParser parser(finder.GetFilePath());
    parser.yyparse();
  }

  if(Option.append)
  {
    // load saved tags
		updatePseudoTags();

    if(!TagFile.classes.isEmpty())
      return;
  }

  Option.ignore.clear();
  for(RegIterator it1(RegExclude);it1;it1++)
    Option.ignore.push_back(CString(it1) + "+");

  for(RegIterator it2(RegExcludeClass);it2;it2++)
    Option.ignore.push_back(CString(it2));

  if(CCommands::Get()->getValue("ParseCommon",1)!=1)
    RefreshCommon();

  AfxTrace("time begin = %f\n", clock());

  buildHashTable();
  buildCKeywordHash();

  // delete tag file from disk
  remove(Option.tagFileName);

  // add comon files in massive
  int idx = CCommands::Get()->getValue("ParseCommon",1);
  ParseState shouldParse = idx==0 ? _ShouldParseCommon : idx==2 ? _MaybeParse : _DontParse;
  for(int p=0;p<Project::commonProject.additionalDirs.size();p++ )
    addCommonFiles(Project::commonProject.additionalDirs[p]+"\\*.*",shouldParse);

  // add all projects in massive
  for (vector<Project>::iterator it = Option.projects.begin(); it!= Option.projects.end();it++)
  {
    Parser parser(it);
    parser.yyparse();
  }

  sortAndUnique(TagFile.m_files);

  // make files list
  {
    for(int i = 0; i<TagFile.m_files.size();i++)
    {
      TagFile.currentFile = i;
      ClassItem classItem;
      classItem.m_name = TagFile.m_files[i].m_name.c_str();
      classItem.m_line = 1;
      writeCtagsEntry(classItem,TAG_FILE);
    }
  }

  TagFile.currentFile = 0;
  bool firstPass = true;
  for(;;)
  {
    vector<int> iparse;
    {
      for(int i = 0; i<TagFile.m_files.size();i++)
      {
        if(firstPass)
        {
          if(TagFile.m_files[i].m_shouldParse==_ShouldParse)
           iparse.push_back(i);
        }
        else 
        {
          if(TagFile.m_files[i].m_shouldParse==_ShouldParse || TagFile.m_files[i].m_shouldParse==_ShouldParseCommon)
           iparse.push_back(i);
        }
      }
    }
    if(iparse.size()==0)
      break;

    for(int i = 0; i<iparse.size();i++)
    {
      int id = iparse[i];
      TagFile.currentFile = id;
      createTagsForFile(TagFile.m_files[id].m_name,LANG_CPP);
      TagFile.m_files[id].m_shouldParse = _AlreadyParsed;
      SetPos(firstPass ? 0: 1, 100*(i)/iparse.size());
      if(GetAsyncKeyState(VK_ESCAPE)<0)
        return;
    }
    firstPass = false;
  }

  AfxTrace("time read = %d\n", clock());

  resortTags();

  AfxTrace("time resort = %d\n", clock());

  writeTagFile();

  AfxTrace("time write = %d\n", clock());

  freeKeywordTable();
}
