// DlgWinMgr.cpp : implementation file
// COPYRIGHT (C) 1999-2000 by Mike Melnikov zmike@andnow.ru
//

#include "stdafx.h"

#include "Commands.h"
#include "TagsOfFileMgr.h"
#include "Utils.h"
#include "tags\read.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern tagFile TagFile;


struct _FileTag
{
  _FileTag(int akind,string acode,string afile,CString aname,int aline): 
    kind(akind),code(acode),file(afile),name(aname),line(aline){}
  int kind;
  string code;
  string file;
  CString name;
  int line;

};

/////////////////////////////////////////////////////////////////////////////
// CTagsOfFileMgr dialog

void CTagsOfFileMgr::OnInit()
{
  m_ctrl.makeColumns("Kind","Line","Tag","Source",0);

  m_ctrl.SendMessage(LVM_SETEXTENDEDLISTVIEWSTYLE,0,(LPARAM)LVS_EX_FULLROWSELECT);

  CImageList* pimagelist = new CImageList();
  pimagelist->Create(16, 10, TRUE/*bMask*/, 6, 4);

  CBitmap	bitmap;
  bitmap.LoadBitmap(IDB_IMAGES);
  pimagelist->Add(&bitmap, (COLORREF)0xFFFFFF);
  bitmap.DeleteObject();

  m_ctrl.SetImageList(pimagelist, TVSIL_STATE);
}

void CTagsOfFileMgr::addItem(string file,int kind,CString name,int line)
{
  if(fileOpen(file.c_str()))
  {
		auto_ptr<char> str(readLine(line));
	  int ind = m_ctrl.InsertItem(-1,"",(long)new _FileTag(kind,str.get(),file,name,line));
		m_ctrl.SetItemState(ind,INDEXTOSTATEIMAGEMASK(1+kind), LVIS_STATEIMAGEMASK);
		m_ctrl.SetItemText(ind,COL_LINE_TAG, itoa(line));
		m_ctrl.SetItemText(ind,COL_TAG_TAG, name);
		m_ctrl.SetItemText(ind,COL_CODE_TAG, str.get());
    fileClose();
  }
}

void CTagsOfFileMgr::addItems(TagVector& vect,CString name,int kind)
{
  for (TagVector::iterator it = vect.begin(); it!= vect.end();it++)
  {
    if(CCommands::Get()->nFile==it->file)
      addItem(TagFile.m_files[it->file].m_name,kind,name,it->line);
  }
}


void CTagsOfFileMgr::addInTree(ClassItem& item,int kind)
{
  if(CCommands::Get()->nFile==item.fileIndex && item.m_line>0)
  {
    addItem((const char*)item.getFile(),kind,item.m_name,item.m_line);
  }
  if(item.m_line==0)
    addItems(item.m_ichildren,item.m_name,kind);
  if(item.fileIndex==-1)
    addItems(item.m_iparents,item.m_name,kind);
	for(int i=0;i<TAG_COUNT;i++)
  	for (ClassItemIter it = item.m_children[i].begin(); it!= item.m_children[i].end();it++)
	  	addInTree(*it,i);
}

void CTagsOfFileMgr::FillList()
{
  {
    for (int i=0;i < m_ctrl.GetItemCount();i++)
       delete (_FileTag*)m_ctrl.GetItemData(i);
    m_ctrl.DeleteAllItems();
  }

  for(int i=0;i<TAG_COUNT;i++)
  {
    for (ClassItemIter it = TagFile.classes.m_children[i].begin(); it!= TagFile.classes.m_children[i].end();it++)
      addInTree(*it,i);
  }

  m_ctrl.SetWidths();
}

int CTagsOfFileMgr::Sort(LPARAM lParam1, LPARAM lParam2)
{
  int iColumn = abs(m_ctrl.m_iSortedCol)-1; // iColumn here is "1 based" so we don't get -0
  int iComp = 0;
  
  _FileTag* val1 = (_FileTag*)lParam1;
  _FileTag* val2 = (_FileTag*)lParam2;

  if(iColumn==COL_KIND_TAG)
	{
		iComp = val1->kind - val2->kind;
  }
  else if(iColumn==COL_LINE_TAG)
  {
		iComp = val1->line - val2->line;
	}
  else if(iColumn==COL_TAG_TAG)
  {
		iComp = val1->name.CompareNoCase(val2->name);
	}
  else if(iColumn==COL_CODE_TAG)
  {
		iComp = CString(val1->code.c_str()).CompareNoCase(val2->code.c_str());
	}
  if(m_ctrl.m_iSortedCol < 0)
		iComp = -iComp; // reverse sorting sense.
	return iComp;
}

void CTagsOfFileMgr::OnSelect(CMListCtrl& ctrl)
{
  for( CListSelIterator iter(ctrl);iter;)
  {
    _FileTag* val = (_FileTag*)m_ctrl.GetItemData(iter);
    CCommands::Get()->SimpleOpenDoc(val->file.c_str(),val->line);
  }
  CDlgMgr::OnSelect(ctrl);
}
                                    
void CTagsOfFileMgr::onDestroy()
{
  for (int i=0;i < m_ctrl.GetItemCount();i++)
     delete (_FileTag*)m_ctrl.GetItemData(i);

  CImageList	*pimagelist;
	pimagelist = m_ctrl.GetImageList(TVSIL_STATE);
	pimagelist->DeleteImageList();
	delete pimagelist;
}
